{-# LANGUAGE CPP #-}
#define PLATFORM_NAME        Windows
#define PLATFORM_PATH        WindowsPath
#define PLATFORM_PATH_SINGLE 'WindowsPath'
#define PLATFORM_STRING      WindowsString
#define PLATFORM_UTF_CODEC   UTF16-LE
#define IS_WINDOWS           1
#include "Include.hs"
