{-# LANGUAGE CPP #-}

#define PLATFORM_NAME        Windows
#define PLATFORM_PATH        WindowsPath
#define PLATFORM_PATH_SINGLE 'WindowsPath'
#define PLATFORM_CHAR        WindowsChar
#define PLATFORM_WORD        Word16
#include "Include.hs"
