{-# LANGUAGE CPP #-}

#define PLATFORM_NAME        Posix
#define PLATFORM_PATH        PosixPath
#define PLATFORM_PATH_SINGLE 'PosixPath'
#define PLATFORM_CHAR        PosixChar
#define PLATFORM_WORD        Word8
#include "Include.hs"
