{-# LANGUAGE CPP #-}
#define PLATFORM_NAME        Posix
#define PLATFORM_PATH        PosixPath
#define PLATFORM_PATH_SINGLE 'PosixPath'
#define PLATFORM_STRING      PosixString
#define IS_WINDOWS           0
#include "Include.hs"
